/*****************************************************************************
*
* File: OsmoseConfiguration.h
*
* Project: Osmose emulator.
*
* Description: This class will handle emulator all things related to Osmose
* configuration, like keyboard config, or default path ect...
*
* Author: Vedder Bruno
* Date: 08/10/2004, 18h00
*
* URL: http://bcz.emu-france.com/
*****************************************************************************/
#ifndef OSMOSE_CONFIGURATION_H
#define OSMOSE_CONFIGURATION_H

#include <SDL/SDL.h>
#include <iostream>
#include <iomanip>
#include <fstream>

//#define CONFIGURATION_VERBOSE

using namespace std;

typedef struct
{
    string key_name;
    int  key_value;
} SdlKeyEntry;

extern SdlKeyEntry key_list[];


class OsmoseConfiguration
{
    public:

        OsmoseConfiguration(const char *ini_file);		// Constructor.
        OsmoseConfiguration();				// Constructor.

        int P1_UP_KEY;
        int P1_DOWN_KEY;
        int P1_LEFT_KEY;
        int P1_RIGHT_KEY;
        int P1_A_KEY;
        int P1_B_KEY;
        int P2_UP_KEY;
        int P2_DOWN_KEY;
        int P2_LEFT_KEY;
        int P2_RIGHT_KEY;
        int P2_A_KEY;
        int P2_B_KEY;
        int GG_START_KEY;
        int SCREENSHOT_KEY;
        int TILESHOT_KEY;
        int SOUNDSHOT_KEY;
        int PAUSE_KEY;
        int QUIT_KEY;
        int DEBUGGER_KEY;

        int JOY1_BUTTON_A;
        int JOY1_BUTTON_B;
        int JOY1_BUTTON_SELECT;


        const string & getUserHomeFolder()
        {
            return user_home_folder;
        }

    private:

        bool loadConfigurationFromFile(const char *f);
        void analyseLine(string &l);
        void reset();

        // Used path Paths
        string default_filename;

        // The folder where Osmose can read/write config, screenshots, soundshots, tileshots, ...
        string user_home_folder;


};

#endif
